@@ -22,12 +22,24 @@ sub get_session_id_from_hit {
 
 1;
 
-__END__
-
 =head1 NAME
 
 Squatting::Mapper - map requests to session queues
 
+=head1 DESCRIPTION
+
+You'll probably never use this module directly, but just so you know...
+The purpose of this module is to be on the lookout for requests that should
+route to controllers that have used the C<Q> subroutine attribute.
+If it encounters such a request, it gives Continuity a $session_id with
+the queue name appeneded to it.  This will cause Continuity to run this
+request in a different session queue.
+
+=head1 SEE ALSO
+
+L<Squatting::Q>,
+L<Continuity::Mapper>
+
 =cut
 
 # Local Variables: ***
